<?php

namespace Drupal\views_fields_on_off\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\filter\InOperator;

/**
 * Provides a handler that adds the form for Fields On/Off.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("views_fields_on_off_form")
 */
class ViewsFieldsOnOffForm extends InOperator {

  /**
   * {@inheritdoc}
   */
  public function canExpose() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function isExposed() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getValueOptions() {
    $all_fields = [];
    foreach ($this->displayHandler->getHandlers('field') as $id => $handler) {
      if ($label = $handler->label()) {
        $all_fields[$id] = $label;
      }
      else {
        $all_fields[$id] = $handler->adminLabel();
      }
    }
    if (isset($this->valueOptions)) {
      return $this->valueOptions;
    }
    $this->valueOptions = $all_fields;

    return $this->valueOptions;
  }

  /**
   * {@inheritdoc}
   */
  public function defineOptions() {
    $options = parent::defineOptions();
    $options['bypass_hook_views_pre_view'] = ['default' => FALSE];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['bypass_hook_views_pre_view'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Bypass hook_views_pre_view()'),
      '#description' => $this->t('Bypassing hook_views_pre_view() means the module will instead reply on hook_preprocess_views_view(),in which case the filter plugin has been fully initialized and one can call a function on the plugin class instead of handling everything in the hook. Because modules like Charts rely on Views Fields On/Off to work earlier in the rendering process, this option is left unchecked by default.'),
      '#default_value' => $this->options['bypass_hook_views_pre_view'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // This is not a real field, and it only affects the query by excluding
    // fields from the display. But Views won't render if the query()
    // method is not present. This doesn't do anything, but it has to be here.
    // This function is a void, so it doesn't return anything.
  }

  /**
   * Theme preprocess function.
   *
   * @see views_fields_on_off_preprocess_views_view()
   *
   * @param array
   *   Theme variables to be rendered.
   */
  public function preprocess(&$variables) {
    $field_options = $this->view->display_handler->getOption('fields');
    foreach ($field_options as $key => &$field) {
      if (isset($this->value[$key]) || in_array($key, $this->value)) {
        continue;
      }
      unset($this->view->field[$key]);
    }
  }
}
